#pragma once

#include <string>

#include <UpdateManagerGlobal.hpp>
#include <ChipUpdateErrors.hpp>

namespace LaCie
{
class IUpdateTask
{
public:
    IUpdateTask() = default;
    IUpdateTask(const IUpdateTask &other) = default;
    IUpdateTask &operator=(const IUpdateTask &other) = default;
    IUpdateTask(IUpdateTask &&t) noexcept = default;
    IUpdateTask &operator=(IUpdateTask &&t) noexcept = default;
    virtual ~IUpdateTask() = default;

    /*!
     * Get update progress of task
     * \return Update progress
    */
    virtual float getProgress() const = 0;

    /*!
     * Get update status of task
     * \return Update status
    */
    virtual TaskStatus getStatus() const = 0;

    /*!
     * Get update result of task
     * \return Update result
    */
    virtual TaskResult getResult() const = 0;

    /*!
     * Start current task execution pipeline
    */
    virtual bool start() = 0;

    /*!
     * Stop current task execution pipeline
    */
    virtual bool stop() = 0;

    /*!
     * Get error code while updating occured flag
     * \return code
     */
    virtual ChipUpdateErrorType errorCode() const = 0;

    /*!
     * Get error description string
     * \return code
     */
    virtual std::string errorDescription() const = 0;

    // Callbacks
    typedef void (*ChangedCallback)(void *);
    virtual bool installStatusChangedClb(ChangedCallback, void *owner) = 0;
    virtual bool installResultChangedClb(ChangedCallback, void *owner) = 0;
    virtual bool installErrorCodeChangedClb(ChangedCallback, void *owner) = 0;
};
}   // namespace LaCie
